function [gam1,se1,kappaT,meanPE1,ntrough]...
    = GMM4GEL(gamma,logc_obs,age,logC,Rs,Rf,instrument,cutoff,grid,lags)
% estimate RRA using MU SDF and standard GMM

% gamma: true relative risk aversion coefficient
% logc_obs: log observed individual consumption
% age: age of agents
% logC: log aggregate consumption
% Rs: matrix of asset returns
% Rf: risk-free rate
% instrument: instrument
% cutoff: age cutoffs (vector of upper bounds of each cohort)
% grid: grid to do grid search for initial value of gamma
% lags: number of lags in Newey-West HAC

N=length(grid); % number of grid points
if N<3
    error('number of grid points must be at least 3')
end

[I,T] = size(logc_obs); % number of agents and periods
T = T-1; % sample size in GMM
K = size(instrument,1)+1; % number of equations
J = size(Rs,1); % number of assets
A = length(cutoff); % number of age cohorts
logrelC = logc_obs-repmat(logC,I,1); % log relative consumption
Rs(:,1)=[]; % asset returns from t = 2 to T
Rf(end)=[]; % risk-free rate from t = 1 to T-1
ER = bsxfun(@minus,Rs,Rf); % excess returns
clear logc_obs Rs Rf

options = optimset('GradObj','on','Display','off','Algorithm','interior-point');

obj = @(gamma)(GMMcrit4GEL(gamma,logrelC,age,logC,ER,instrument,cutoff,lags));
% first, do a grid search on a coarse grid to get initial value (optional)
if nargout==5
crit=zeros(1,N); % GMM criterion at grid points
for n=1:N
    crit(n)=obj(grid(n));
end
critSign = sign(diff(crit)); % sign of difference of criterion; 1 if increasing, -1 if decreasing
ntrough = sum(critSign(2:end)>critSign(1:end-1)); % number of troughs in grid search
[~,ind]=min(crit);
gam0=grid(ind); % minimizer over grid
clear crit critSign ind
else gam0=gamma; % set initial value to true gamma
end
[gam1,fval] = fmincon(obj,gam0,[],[],[],[],0,[],[],options);
% minimize GMM criterion using gam0 as initial value
[gT,dgT] = perror4(gam1,logrelC,age,logC,ER,instrument,cutoff); % pricing error and its derivative
fT = residual4(gam1,logrelC,age,logC,ER,instrument,cutoff);
S1 = NeweyWestHAC(fT,lags,1);
var1 = (1/T)*pinv(dgT'*(S1\dgT));
se1 = sqrt(diag(var1));
meanPE1 = sqrt((gT'*gT)/(K*J*A)); % mean squared pricing error
kappaT = -2*T/(2*lags+1)*log(-fval);
end