% conduct hypothesis tests of GMM estimation
% load simulation results before running, as below
%load runM10000T500S1

% specification test using mean squared pricing error; essentially the same
% as the bootstrap test in Toda & Walsh (2014)
q95PE_std = quantile(results_std(4,:),0.95); % 95 percentile of standard GMM pricing error
rejectRatePE_stdF = sum(results_stdF(4,:)>q95PE_std)/M; % rejection rate of standard GMM with false returns
q95PE_stdGEL = quantile(results_stdGEL(4,:),0.95); % 95 percentile of standard GEL-GMM pricing error
rejectRatePE_stdGELF = sum(results_stdGELF(4,:)>q95PE_stdGEL)/M; % rejection rate of standard GEL-GMM with false returns

q95PE_age = quantile(results_age(4,:),0.95); % 95 percentile of age cohort GMM pricing error
rejectRatePE_ageF = sum(results_ageF(4,:)>q95PE_age)/M; % rejection rate of age cohort GMM with false returns

q95PE_RA = quantile(results_RA(4,:),0.95); % 95 percentile of representative-agent GMM pricing error
rejectRatePE_RAF = sum(results_RAF(4,:)>q95PE_RA)/M; % rejection rate of representative-agent GMM with false returns
q95PE_RAGEL = quantile(results_RAGEL(4,:),0.95); % 95 percentile of representative-agent GEL-GMM pricing error
rejectRatePE_RAGELF = sum(results_RAGELF(4,:)>q95PE_RAGEL)/M; % rejection rate of representative-agent GEL-GMM with false returns

% J test for true and false models
% For true model, gives size distortions
% For false model, gives Type II error rate
q95J1_std = quantile(results_std(3,:),0.95); % 95 percentile of standard GMM J1 statistic
rejectRateJ1Sim_stdF = sum(results_stdF(3,:)>q95J1_std)/M; % rejection rate of standard GMM with false returns
q95J1_stdGEL = quantile(results_stdGEL(3,:),0.95); % 95 percentile of standard GEL-GMM J1 statistic
rejectRateJ1Sim_stdGELF = sum(results_stdGELF(3,:)>q95J1_stdGEL)/M; % rejection rate of standard GEL-GMM with false returns

q95J1_age = quantile(results_age(3,:),0.95); % 95 percentile of age cohort GMM J1 statistic
rejectRateJ1Sim_ageF = sum(results_ageF(3,:)>q95J1_age)/M; % rejection rate of age cohort GMM with false returns

q95J1_RA = quantile(results_RA(3,:),0.95); % 95 percentile of representative-agent GMM J1 statistic
rejectRateJ1Sim_RAF = sum(results_RAF(3,:)>q95J1_RA)/M; % rejection rate of representative-agent GMM with false returns
q95J1_RAGEL = quantile(results_RAGEL(3,:),0.95); % 95 percentile of representative-agent GEL-GMM J1 statistic
rejectRateJ1Sim_RAGELF = sum(results_RAGELF(3,:)>q95J1_RAGEL)/M; % rejection rate of representative-agent GEL-GMM with false returns

% K = size(Rs,1)*(1+size(instrument,1)); % number of equations
K = 2; 
rejectRateJ1_std = sum(1-chi2cdf(results_std(3,:),K-1)<0.05)/M;
% rejection rate of standard GMM with true asset returns
rejectRateJ1_stdGEL = sum(1-chi2cdf(results_stdGEL(3,:),K-1)<0.05)/M;
% rejection rate of standard GEL-GMM with true asset returns
rejectRateJ1_age = sum(1-chi2cdf(results_age(3,:)/Ncut,K-1)<0.05)/M;
% rejection rate of age cohort GMM with true asset returns
rejectRateJ1_RA = sum(1-chi2cdf(results_RA(3,:),K-1)<0.05)/M;
% rejection rate of representative-agent GMM with true asset returns
rejectRateJ1_RAGEL = sum(1-chi2cdf(results_RAGEL(3,:),K-1)<0.05)/M;
% rejection rate of representative-agent GEL-GMM with true asset returns
rejectRateJ1_stdF = sum(1-chi2cdf(results_stdF(3,:),K-1)<0.05)/M;
% rejection rate of standard GMM with false asset returns
rejectRateJ1_stdGELF = sum(1-chi2cdf(results_stdGELF(3,:),K-1)<0.05)/M;
% rejection rate of standard GEL-GMM with false asset returns
rejectRateJ1_ageF = sum(1-chi2cdf(results_ageF(3,:)/Ncut,K-1)<0.05)/M;
% rejection rate of age cohort GMM with false asset returns
rejectRateJ1_RAF = sum(1-chi2cdf(results_RAF(3,:),K-1)<0.05)/M;
% rejection rate of representative-agent GMM with false asset returns
rejectRateJ1_RAGELF = sum(1-chi2cdf(results_RAGELF(3,:),K-1)<0.05)/M;
% rejection rate of representative-agent GEL-GMM with false asset returns

% t-test of gam1 = gamma; gives Type I error rate
TypeIerrT1_std = sum(1-normcdf(abs(results_std(1,:)-gamma)./results_std(2,:))<0.025)/M;
TypeIerrT1_stdGEL = sum(1-normcdf(abs(results_stdGEL(1,:)-gamma)./results_stdGEL(2,:))<0.025)/M;
TypeIerrT1_age = sum(1-normcdf(abs(results_age(1,:)-gamma)./results_age(2,:))<0.025)/M;
TypeIerrT1_RA = sum(1-normcdf(abs(results_RA(1,:)-gamma)./results_RA(2,:))<0.025)/M;
TypeIerrT1_RAGEL = sum(1-normcdf(abs(results_RAGEL(1,:)-gamma)./results_RAGEL(2,:))<0.025)/M;

Gam1 = [results_std(1,:)' results_age(1,:)' results_RA(1,:)'...
    results_stdF(1,:)' results_ageF(1,:)' results_RAF(1,:)']; % gamma1
SE1 = [results_std(2,:)' results_age(2,:)' results_RA(2,:)'...
    results_stdF(2,:)' results_ageF(2,:)' results_RAF(2,:)']; % gamma1
ind1 = find(sum(isnan(Gam1),2)==0);
Gam1 = Gam1(ind1,:);
SE1 = SE1(ind1,:);
SE1trunc100 = min(SE1,100); % standard error truncated at 100
%Gam2 = Gam2(ind2,:);
bias1 = mean(Gam1)-gamma; % bias of first stage gamma estimate
%bias2 = mean(Gam2)-gamma; % bias of second stage gamma estimate
se1 = mean(SE1); % mean standard error
se1trunc100 = mean(SE1trunc100); % mean standard error truncated at 100
MAE1 = mean(abs(Gam1-gamma)); % mean absolute error
%MAE2 = mean(abs(Gam2-gamma));
RMSE1 = sqrt(mean((Gam1-gamma).^2)); % root mean squared error
%RMSE2 = sqrt(mean((Gam2-gamma).^2));

% count the number of times with multiple troughs
multTroughs = sum([results_std(5,:)' results_age(5,:)' results_RA(5,:)']>1);

% count the number of times with pricing error less than 0.01, 0.001,
% 0.0001
PE001 = sum([results_stdF(4,:)' results_ageF(4,:)' results_RAF(4,:)']<=1e-2);
PE0001 = sum([results_stdF(4,:)' results_ageF(4,:)' results_RAF(4,:)']<=1e-3);
PE00001 = sum([results_stdF(4,:)' results_ageF(4,:)' results_RAF(4,:)']<=1e-4);

% output results
bias1 % bias of first stage gamma estimate
se1 % mean standard error
se1trunc100 % mean standard error truncated at 100
MAE1 % mean absolute error
RMSE1 % mean squared error
rejectRatePESim_false = [rejectRatePE_stdF rejectRatePE_ageF rejectRatePE_RAF];
rejectRateJ1Sim_false = [rejectRateJ1Sim_stdF rejectRateJ1Sim_ageF rejectRateJ1Sim_RAF];
rejectRateJ1_true = [rejectRateJ1_std rejectRateJ1_age rejectRateJ1_RA];
rejectRateJ1_false = [rejectRateJ1_stdF rejectRateJ1_ageF rejectRateJ1_RAF];
TypeIerrT1 = [TypeIerrT1_std TypeIerrT1_age TypeIerrT1_RA]
TypeIerrJ1 = rejectRateJ1_true
TypeIIerrPESim = 1-rejectRatePESim_false
TypeIIerrJ1Sim = 1-rejectRateJ1Sim_false
TypeIIerrJ1 = 1-rejectRateJ1_false

% Output GEL results
Gam1GEL = [results_stdGEL(1,:)' results_RAGEL(1,:)'...
    results_stdGELF(1,:)' results_RAGELF(1,:)']; % gamma1
SE1GEL = [results_stdGEL(2,:)' results_RAGEL(2,:)'...
    results_stdGELF(2,:)' results_RAGELF(2,:)']; % gamma1
ind1GEL = find(sum(isnan(Gam1GEL),2)==0);
Gam1GEL = Gam1GEL(ind1GEL,:);
SE1GEL = SE1GEL(ind1GEL,:);
SE1trunc100GEL = min(SE1GEL,100); % standard error truncated at 100
bias1GEL = mean(Gam1GEL)-gamma; % bias of first stage gamma estimate
se1GEL = mean(SE1GEL); % mean standard error
se1trunc100GEL = mean(SE1trunc100GEL); % mean standard error truncated at 100
MAE1GEL = mean(abs(Gam1GEL-gamma)); % mean absolute error
RMSE1GEL = sqrt(mean((Gam1GEL-gamma).^2)); % root mean squared error

% count the number of times with multiple troughs
multTroughsGEL = sum([results_stdGEL(5,:)' results_RAGEL(5,:)']>1);

% output results
bias1GEL % bias of first stage gamma estimate
se1GEL % mean standard error
se1trunc100GEL % mean standard error truncated at 100
MAE1GEL % mean absolute error
RMSE1GEL % mean squared error
rejectRatePESim_falseGEL = [rejectRatePE_stdGELF rejectRatePE_RAGELF];
rejectRateJ1Sim_falseGEL = [rejectRateJ1Sim_stdGELF rejectRateJ1Sim_RAGELF];
rejectRateJ1_trueGEL = [rejectRateJ1_stdGEL rejectRateJ1_RAGEL];
rejectRateJ1_falseGEL = [rejectRateJ1_stdGELF rejectRateJ1_RAGELF];
TypeIerrT1GEL = [TypeIerrT1_stdGEL TypeIerrT1_RAGEL]
TypeIerrJ1GEL = rejectRateJ1_trueGEL
TypeIIerrPESimGEL = 1-rejectRatePESim_falseGEL
TypeIIerrJ1SimGEL = 1-rejectRateJ1Sim_falseGEL
TypeIIerrJ1GEL = 1-rejectRateJ1_falseGEL